//PluginGUI
//by:	I.Robinson, PassMark Software
//Copyright 2009-2015
//
//Purpose:
//	BurnInTest Plugin GUI
//
//History:
// 4 May 2015: v2.0 1000: I.Robinson - Correction for use as pre-test plugin and VS2013
// 11 March 2009: v1.0 1000: I.Robinson - First version

#define _WIN32_WINNT 0x0400
#define WIN32_LEAN_AND_MEAN
#define	MAIN_MODULE		//Set flag, used in header file for global vars
						//This flag must only be set in one module

#define MAXCOMMANDLINE			256

#define PLUGIN_WRITE "Plug-in write"
#define PLUGIN_READ  "Plug-in read"
#define PLUGIN_WAIT  "Plug-in sleep"

#include <windows.h>
#include <commctrl.h>
#include <shellapi.h>
#include <stdio.h>
#include <process.h>
#include <time.h>
#include <stdlib.h>
#include "resource.h"
#include "PluginGUI.h"
#include "PluginTestInterface.h"

//
//Function protypes
//
LRESULT CALLBACK WndProc (HWND, UINT, WPARAM, LPARAM) ;
void InitDialog (HWND hwnd);
void OnPaint (HWND hwnd);
void OnAction1 (HWND hwnd);
void OnAction2 (HWND hwnd);
void HandleUserAction1();
void HandleUserAction2();
bool StartPluginTest();
DWORD WINAPI StartPluginTestThread(LPVOID Param);
void strn_clean_cpy (char * szTextOut, char * szTextIn, int iStrLen);

//
//Globals for this module
//
static int		fTestRunning;		//T if test is running.
static char		SharedMemInterface[50];
static HANDLE	hMappedObject;
static char		g_szSharedMem[MAXCOMMANDLINE] = {'\0'};
static PLUGININTERFACE	*BitInterface;
static int		BufferPattern;
static UINT		Cycle;
static bool		bErrorCondition = false;
static int		iNumWrites = 100;		//for demonstration purposes
static int		i;
static int		iTestPhase;
static char		szTemp[PLUGIN_MAXDISPLAYTEXT];
static bool		bSetLabels = false;
static bool		g_UserAction1 = false;
static bool		g_UserAction2 = false;

#ifdef _DEBUG
char szTempDebug[1024] = {"\0"};
#endif

//********************************
//Main windows procedure
//
//Code entry point
//********************************
int WINAPI WinMain (HINSTANCE hInst, HINSTANCE hPrevInstance,
                    PSTR szCmdLine, int iCmdShow)
{
	static char  szAppName[] = "PluginGUI" ;
	MSG          msg ;
	WNDCLASSEX   wndclass ;

	hInstance = hInst;

	//Get the applications directory
	GetModuleFileName (NULL, fullpath, sizeof(fullpath));
	*strrchr (fullpath, '\\') = '\0';
	strcpy (dirname, fullpath);

	//Set the tests to do by default, pass the name of the config file
	if (!ParseCommandLine (szCmdLine))
	{
#ifdef _DEBUG
		MessageBox(NULL, "ParseCommandLine failed", "ERROR", MB_OK | MB_ICONERROR);
#endif
		return (1);
	}

	wndclass.cbSize        = sizeof (wndclass) ;
	wndclass.style         = CS_HREDRAW | CS_VREDRAW;
	wndclass.lpfnWndProc   = WndProc ;
	wndclass.cbClsExtra    = 0 ;
	wndclass.cbWndExtra    = DLGWINDOWEXTRA ;
	wndclass.hInstance     = hInstance ;
	wndclass.hIcon         = LoadIcon (hInstance, szAppName) ;
	wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
	wndclass.hbrBackground = (HBRUSH) (COLOR_BTNFACE +1) ;
	wndclass.lpszMenuName  = NULL ;
	wndclass.lpszClassName = szAppName ;
	wndclass.hIconSm       = LoadIcon (hInstance, szAppName) ;
	
	RegisterClassEx (&wndclass) ;
	
	//Create and display the window	
	main_hwnd = CreateDialog (hInstance, szAppName, 0, NULL) ;
	InitDialog (main_hwnd);
	ShowWindow (main_hwnd, iCmdShow) ;

	//Start Plugin Test
	StartPluginTest();

	//Handle windows messages (message pump)
	while (GetMessage (&msg, NULL, 0, 0))
	{
		TranslateMessage (&msg) ;
		DispatchMessage (&msg) ;
	}
	return msg.wParam ;
}


//********************************
//Windows callback procedure
//
//Handles all the messages for the dialog box
//********************************
LRESULT CALLBACK WndProc (HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam) {

	
	switch (iMsg)
	{
	
	case WM_CREATE :
		return 0 ;

	case WM_SHOWWINDOW :
		return 0 ;

	case WM_PAINT :
		OnPaint (hwnd);
        break ;

	case WM_COMMAND :
		switch (LOWORD (wParam)) {
			case IDC_BUTTON_EXIT:
				DestroyWindow (hwnd);
				break;
			case IDCONFIG:
				break;
			case IDHELP:
				break;
			case IDACTION1:
				OnAction1 (hwnd);
				break;
			case IDACTION2:
				OnAction2 (hwnd);
				break;
			case IDSAVELOG:
				break;
			case IDCLEAR:
				break;
			case IDC_LOG:
				break;
		}
		return 0 ;
		
	case WM_DESTROY :
		//Signal sutdown to BurnIntest
		BitInterface->IN_TestRunning = false; 
		BitInterface->OUT_iStatus = PRE_TEST_PLUGIN_COMPLETED;

		//Wait for BurnINtest to read the complete message
		int wait_time = 1000;
		while (wait_time > 0)
		{
			Sleep(100);
			wait_time -= 100;
			if (!BitInterface->IN_TestRunning)
				break;
		}

		PostQuitMessage (0) ;
		EndDialog(hwnd, LOWORD(wParam));
		return 0;
	}
	return DefWindowProc (hwnd, iMsg, wParam, lParam) ;
}


//********************************
//StartPluginTest
//
//Starts the Plugin test
//********************************
bool StartPluginTest()
{
	//Init plugin interface
	//Get name of shared memory
	if (g_szSharedMem[0] != 'B' && g_szSharedMem[1] != 'I')
	{
#ifdef _DEBUG
		sprintf(szTempDebug,"(g_szSharedMem[0] != 'B' && g_szSharedMem[1] != 'I') failed: %s", g_szSharedMem);
		//MessageBox(NULL, szTempDebug, "ERROR", MB_OK | MB_ICONERROR);
#endif
		return false;
	}

	//Open shared memory interface
	hMappedObject = OpenFileMapping (FILE_MAP_ALL_ACCESS, FALSE, g_szSharedMem);
	if (hMappedObject == NULL)
	{
#ifdef _DEBUG
		MessageBox(NULL, "OpenFileMapping failed", "ERROR", MB_OK | MB_ICONERROR);
#endif
		return false;
	}

	BitInterface = (PLUGININTERFACE *) MapViewOfFile(hMappedObject,FILE_MAP_ALL_ACCESS,0,0,0);
	if (BitInterface == NULL) {
		CloseHandle (hMappedObject);
#ifdef _DEBUG
		MessageBox(NULL, "MapViewOfFile failed", "ERROR", MB_OK | MB_ICONERROR);
#endif
		return false;
	}

	//Start Plugin Test thread
	DWORD	dwSendThreadID = 0;
	CreateThread (NULL, 0, StartPluginTestThread, 0, 0, &dwSendThreadID);

	return true;

}

//********************************
//StartPluginTestThread
//
//Starts the Plugin test as a software thread (allows the GUI messages to still be handled)
//********************************
DWORD WINAPI StartPluginTestThread(LPVOID Param) 
{

	//Set initial test values
	BitInterface->OUT_Plugin_interface_version = PLUGIN_INTERFACE_VERSION;
	BitInterface->OUT_iCycle = 0;
	BitInterface->OUT_iStatus = 0;
	BitInterface->OUT_iErrorCount = 0;
	BitInterface->OUT_iErrorSeverity = 0;
	BitInterface->OUT_i64WriteOps = 0;//PLUGIN_NOTUSED;
	BitInterface->OUT_i64ReadOps = 0;//PLUGIN_NOTUSED;
	BitInterface->OUT_i64VerifyOps = 0; //PLUGIN_NOTUSED;
	
	strn_clean_cpy(BitInterface->OUT_szUserDefVal1,"0 apples", PLUGIN_MAXDISPLAYTEXT);	
	BitInterface->OUT_bUserDef1 = true;
	BitInterface->OUT_bNewUserDefVal1 = true;

	strn_clean_cpy(BitInterface->OUT_szUserDefVal2,"0 oranges", PLUGIN_MAXDISPLAYTEXT);	
	BitInterface->OUT_bUserDef2 = true;
	BitInterface->OUT_bNewUserDefVal2 = true;

	BitInterface->OUT_bNewError = false;		
	BitInterface->OUT_bNewStatus = false;		
	BitInterface->OUT_bTestStopped = false;	

	// Set the labels that will be displayed on the BurnInTest Plugin Test window
	strn_clean_cpy(BitInterface->OUT_szWindowTitle,"Test plugin", PLUGIN_MAXDISPLAYTEXT);	
	strn_clean_cpy(BitInterface->OUT_szStatus,"Starting", PLUGIN_MAXDISPLAYTEXT);				
	strn_clean_cpy(BitInterface->OUT_szError,"", PLUGIN_MAXERRORTEXT);						
	strn_clean_cpy(BitInterface->OUT_szWriteOps, "Write (MBytes):", PLUGIN_MAXDISPLAYTEXT);						
	strn_clean_cpy(BitInterface->OUT_szReadOps,"Read (MBytes):", PLUGIN_MAXDISPLAYTEXT);							
	strn_clean_cpy(BitInterface->OUT_szVerifyOps, "Verify (MBytes):", PLUGIN_MAXDISPLAYTEXT);
	strn_clean_cpy(BitInterface->OUT_szUserDef1,"Apples:", PLUGIN_MAXDISPLAYTEXT);				
	strn_clean_cpy(BitInterface->OUT_szUserDef2, "Oranges:", PLUGIN_MAXDISPLAYTEXT);	
	BitInterface->OUT_bDisplayTextSet = true;
		

	//Allocate RAM
	Cycle = 0;
	iTestPhase = 1;

	//Run the test until BurnInTest flags that the test has been stopped
	//Write , read and verify test data in a loop
	// WARNING: Never perform any activity that takes more than 1 second without 
	// checking the stop condition IN_TestRunning
	while (BitInterface->IN_TestRunning) 
	{
		//////////////////////////////////////////
		//Write
		//////////////////////////////////////////
		BitInterface->OUT_iStatus = PLUGIN_WRITING;
		strn_clean_cpy(BitInterface->OUT_szStatus, PLUGIN_WRITE, PLUGIN_MAXDISPLAYTEXT);
		BitInterface->OUT_bNewStatus = true;		//flag that a new status string is available

		// .... insert any write operations here 
		// eg. loop writing to disk
		for (i=0;i<iNumWrites;i++)
		{
			Sleep(10);								//insert Write operation
			
			BitInterface->OUT_i64WriteOps += 1;
			
			if (iTestPhase  == 1)
			{
				sprintf(szTemp,"%d apples", BitInterface->OUT_i64WriteOps);
				strn_clean_cpy(BitInterface->OUT_szUserDefVal1,szTemp, PLUGIN_MAXDISPLAYTEXT);	//Note: you may wait to ensure BIT has read the old value
				BitInterface->OUT_bNewUserDefVal1 = true;
			}
			else if (iTestPhase  == 2)
			{
				sprintf(szTemp,"%d apes", BitInterface->OUT_i64WriteOps);
				strn_clean_cpy(BitInterface->OUT_szUserDefVal1,szTemp, PLUGIN_MAXDISPLAYTEXT);	//Note: you may wait to ensure BIT has read the old value
				BitInterface->OUT_bNewUserDefVal1 = true;
			}

		}
		// ........................................

		//////////////////////////////////////////
		//Read
		//////////////////////////////////////////
		BitInterface->OUT_iStatus = PLUGIN_READING;
		strn_clean_cpy(BitInterface->OUT_szStatus, PLUGIN_READ ,PLUGIN_MAXDISPLAYTEXT);
		BitInterface->OUT_bNewStatus = true;		//flag that a new status string is available

		// ...... insert any read operations here .
		// eg. loop reading from disk
		for (i=0;i<iNumWrites;i++)
		{
			Sleep(10);								//insert Write operation

			BitInterface->OUT_i64ReadOps  += 1;

					
			
			if (iTestPhase  == 1)
			{
				sprintf(szTemp,"%d oranges", BitInterface->OUT_i64ReadOps);
				strn_clean_cpy(BitInterface->OUT_szUserDefVal2,szTemp, PLUGIN_MAXDISPLAYTEXT);	//Note: you may wait to ensure BIT has read the old value
				BitInterface->OUT_bNewUserDefVal2 = true;
			}
			else if (iTestPhase  == 2)
			{
				sprintf(szTemp,"%d monkeys", BitInterface->OUT_i64ReadOps);
				strn_clean_cpy(BitInterface->OUT_szUserDefVal2,szTemp, PLUGIN_MAXDISPLAYTEXT);	//Note: you may wait to ensure BIT has read the old value
				BitInterface->OUT_bNewUserDefVal2 = true;
			}
		}

		//////////////////////////////////////////
		//Verify
		//////////////////////////////////////////
		BitInterface->OUT_iStatus = PLUGIN_VERIFYING;

		// ...... insert any read operations here .
		// eg. loop verifying data read from disk
		BitInterface->OUT_i64VerifyOps += 1;

		bErrorCondition = true; //just for demonstration

		if (bErrorCondition)
		{
			//Allow BIT a chance to clear the last error, before passing a new one
			while (BitInterface->IN_TestRunning && BitInterface->OUT_bNewError)
				Sleep (500);	
			
			BitInterface->OUT_iErrorCount++;
			BitInterface->OUT_iErrorSeverity = ERRORINFORMATION;
			strn_clean_cpy(BitInterface->OUT_szError,
							"Plugin specified error message. This may be up to 100 characters long ABCDEFGHIJKLMNOPQRSTUVWXYZ", 
							PLUGIN_MAXERRORTEXT);
			BitInterface->OUT_bNewError = true;
		}

		//////////////////////////////////////////
		//Check for user manual input - such as checking that a hardware LED came on on the hardware
		//////////////////////////////////////////
		HandleUserAction1();
		HandleUserAction2();

		//////////////////////////////////////////
		//Cycle finished
		//////////////////////////////////////////
		BitInterface->OUT_iStatus = PLUGIN_WAITING;
		strn_clean_cpy(BitInterface->OUT_szStatus, PLUGIN_WAIT, PLUGIN_MAXDISPLAYTEXT);
		BitInterface->OUT_bNewStatus = true;		//flag that a new status string is available

		BitInterface->OUT_iCycle += 1;	//Update the BIT cycle counter
		

		// Introduc a duty cycle delay (in this case between cycles)
		if (BitInterface->IN_TestRunning)
			Sleep ((100 - BitInterface->IN_DutyCycle) * 20); //For demonstartion purposes. The Duty cycle delay could be inserted anywhere.


		//Check for a change in the test phase. As the test window labels may need to change.
		// Change the test labels for the next type of test
		// Don't write new values until BIT has flagged that it has read the old ones.
		if (BitInterface->OUT_iCycle == 10 && iTestPhase == 1) //10 is just an arbitrary number of cycles for the first test phase
		{
			iTestPhase = 2;
			bSetLabels = false;
			while (BitInterface->IN_TestRunning && !bSetLabels) 
			{
				if (BitInterface->OUT_bDisplayTextSet == false)	//Previous labels not yet read by BIT
				{
					strn_clean_cpy(BitInterface->OUT_szWindowTitle,"Test plugin2", PLUGIN_MAXDISPLAYTEXT);	
					strn_clean_cpy(BitInterface->OUT_szStatus,"Eating Bannanas", PLUGIN_MAXDISPLAYTEXT);				
					strn_clean_cpy(BitInterface->OUT_szWriteOps, "Bannanas Grown:", PLUGIN_MAXDISPLAYTEXT);						
					strn_clean_cpy(BitInterface->OUT_szReadOps,"Banannas Eaten:", PLUGIN_MAXDISPLAYTEXT);							
					strn_clean_cpy(BitInterface->OUT_szVerifyOps, "Bannanas enjoyed:", PLUGIN_MAXDISPLAYTEXT);
					strn_clean_cpy(BitInterface->OUT_szUserDef1,"Gorillas happy:", PLUGIN_MAXDISPLAYTEXT);				
					strn_clean_cpy(BitInterface->OUT_szUserDef2, "Gorillas sad:", PLUGIN_MAXDISPLAYTEXT);	
					BitInterface->OUT_bUserDef2 = false;		//User defined value 2 not used for test phase 2 
					BitInterface->OUT_bDisplayTextSet = true;	//signal to BIT that the labels have been updated and that they should be changed
					bSetLabels = true;
				}
				else
					Sleep(1000);
			}
		}
	}

	return 0;
}

//********************************
//OnAction1
//
//********************************
void OnAction1 (HWND hwnd) 
{
	g_UserAction1 = true;
}


//********************************
//OnAction2
//
//Start the test
//********************************
void OnAction2 (HWND hwnd) 
{
	g_UserAction2 = true;
}


//EnableButtons
//
//Enables or disables the buttons when the test starts or stops
//state = FALSE == Disable buttons
//********************************
void EnableButtons (HWND hwnd, int state) {
	
	EnableWindow (GetDlgItem(hwnd,IDSTART) ,state);
	EnableWindow (GetDlgItem(hwnd,IDCONFIG) ,state);
	EnableWindow (GetDlgItem(hwnd,IDMODEMINFO) ,state);
	EnableWindow (GetDlgItem(hwnd,IDHELP) ,state);
	EnableWindow (GetDlgItem(hwnd,IDABOUT) ,state);
}

			 
//********************************
//InitDialog
//
//Called just after the dialog is created
//********************************
void InitDialog (HWND hwnd) {

	//Set up defaults
	fTestRunning = false;
}



//********************************
//OnPaint
//
//Paints the dialog
//********************************
void OnPaint (HWND hwnd) {
}

//********************************
//ValidPathName
//
//Checks that a path:
//	has length > 0
//  does not contain: /*?<>|
//returns:
//	true if valid
//	false if invlaid
//********************************
bool ValidPathName (char *Path) 
{

		if ((strlen(Path) == 0) || 
			(strrchr( Path, '/' ) != 0) ||
			(strrchr( Path, '*' ) != 0) ||
			(strrchr( Path, '?' ) != 0) ||
			(strrchr( Path, '<' ) != 0) ||
			(strrchr( Path, '>' ) != 0) ||
			(strrchr( Path, '|' ) != 0) ) 
			return false;
		else
			return true;
}


//Name:	ParseCommandLine
//Function:
//	Reads in the command line
//Notes
//  Called only once when the program starts up
//	No command line arguments = No auto testing
bool ParseCommandLine (char *commandline) {

	int		CmdLineIndex;

#ifdef _DEBUG
	sprintf(szTempDebug,"ParseCommandLine: %d", __argc);
	//MessageBox(NULL, szTempDebug, "ERROR", MB_OK | MB_ICONERROR);
#endif

	//Check for command line options
	for(CmdLineIndex = 0; CmdLineIndex < __argc; CmdLineIndex++)
	{
#ifdef _DEBUG
		sprintf(szTempDebug,"ParseCommandLine: %s", __argv[CmdLineIndex]);
		//MessageBox(NULL, szTempDebug, "ERROR", MB_OK | MB_ICONERROR);
#endif
		if (strstr(__argv[CmdLineIndex], "BIT_PLUGIN_INT"))
			strcpy(g_szSharedMem, __argv[CmdLineIndex]);
	}

	return true;
}

//strn_clean_cpy 
// 
// Use this function to cpy any string to the BurnInTest Interface to ensure bonudary lengths are not exceeded
//
// does a strncpy, but cleans the string up first (removes funny characters, '%' and '\' and make them spaces)
//
void strn_clean_cpy (char * szTextOut, char * szTextIn, int iStrLen)
{
	char	szTextTemp[PLUGIN_MAXERRORTEXT+1];
	bool	bEndString = false;

	if (iStrLen > PLUGIN_MAXERRORTEXT)
		iStrLen  = PLUGIN_MAXERRORTEXT;

	for (int i = 0; i < iStrLen; i++)
	{
		if (szTextIn[i] == '\0' || bEndString)
		{
			szTextTemp[i] = '\0';
			bEndString = true;
		}
		else if (szTextIn[i] < 0x020)
			szTextTemp[i] = ' ';
		else if (szTextIn[i] == '%')
			szTextTemp[i] = ' ';
		else if (szTextIn[i] == '\\')
			szTextTemp[i] = ' ';
		else
			szTextTemp[i] = szTextIn[i];
	}
	szTextTemp[iStrLen-1] = '\0'; //Make sure its terminated

	strncpy (szTextOut, szTextTemp, iStrLen);
	szTextOut[iStrLen-1] = '\0'; 
}


void HandleUserAction1()
{
	if (g_UserAction1)
	{
		//Allow BIT a chance to clear the last error, before passing a new one
		while (BitInterface->IN_TestRunning && BitInterface->OUT_bNewError)
			Sleep (500);	

		BitInterface->OUT_iErrorCount++;
		BitInterface->OUT_iErrorSeverity = ERRORINFORMATION;
		strn_clean_cpy(BitInterface->OUT_szError,
			"User action error 1. This may be up to 100 characters long ABCDEFGHIJKLMNOPQRSTUVWXYZ", 
			PLUGIN_MAXERRORTEXT);
		BitInterface->OUT_bNewError = true;

		g_UserAction1 = false; //reset
	}
}


void HandleUserAction2()
{
	if (g_UserAction2)
	{
		//Allow BIT a chance to clear the last error, before passing a new one
		while (BitInterface->IN_TestRunning && BitInterface->OUT_bNewError)
			Sleep (500);	

		BitInterface->OUT_iErrorCount++;
		BitInterface->OUT_iErrorSeverity = ERRORINFORMATION;
		strn_clean_cpy(BitInterface->OUT_szError,
			"User action error 2. This may be up to 100 characters long ABCDEFGHIJKLMNOPQRSTUVWXYZ", 
			PLUGIN_MAXERRORTEXT);
		BitInterface->OUT_bNewError = true;

		g_UserAction2 = false; //reset
	}
}
